# MT::Plugins::Redland::CMS::Topic::Image
# -----------------
# $Revision: 1.3 $
# $Date: 2005/04/04 10:12:45 $
# ---------------------------------------------------------

package MT::Plugins::Redland::CMS::Topic::Image;

use strict;
use warnings;
no warnings 'once';

use Carp qw(confess);
use IO::All;
use File::Spec;
use Data::Dumper;
use RDF::Redland;

use MT 2.6;
use MT::Template::Context;

our ($NAME, $VERSION, $plugin_data, $debug);
BEGIN {
	no warnings 'numeric';
	$NAME			= "Redland Image Topic";
	$VERSION		= do { my @REV = split(/\./, (qw$Revision: 1.3 $)[1]); sprintf("%0.3f", $REV[0] + ($REV[1]/1000)) };
	$debug			= 0;
	$plugin_data	= {
		name		=> "${NAME}, v${VERSION}",
		description => 'Adds an image topic to an Entry',
		doc_link	=> 'http://kasei.us/code/mtredland/'
	};
	
	if ($MT::VERSION >= 3.0) {
		require MT::Plugin;
		MT::Plugin->import();
		our $plugin	= new MT::Plugin($plugin_data);
		MT->add_plugin($plugin);
	}
	my $cfg		= MT::ConfigMgr->instance;
	my $cms		= File::Spec->catfile( $cfg->PluginPath, 'redland_cms_topic.pl' );
	require $cms;
}

######################################################################

MT::Plugins::Redland::CMS->install_cms_header( ${NAME}, <<"END" );
<style type="text/css">
	#Image input { width: 300px; }
</style>
END
MT::Plugins::Redland::CMS::Topic->install_topic( 'Image', sub {
	my $app		= shift;
	my $q		= $app->{query};
	my $id		= $q->param('id');
	my $type	= $q->param('_type');
	my $class	= $app->_load_driver_for($type) or return;
	my $url		= '';
	if ($id) {          # object exists, we're just editing it.
		my $entry	= $class->load($id) or return $app->error($app->translate("Load failed: [_1]", $class->errstr || "(no reason given)"));
		$url		= $entry->image_url;
	}
	return qq(<p><label for="image_url">URL</label>: <input name="image_url" id="image_url" value="${url}" /></p>);
});

no warnings 'redefine';
my $orig	= MT::Entry->can('column_names');
*MT::Entry::column_names	= sub {
	my $self	= shift;
	my $names	= $orig->( $self, @_ );
	push(@{ $names }, 'image_url');
	return $names;
};

MT::Entry->add_callback("post_load", 4, $plugin_data, sub {
	my ($cb, $terms, $args, $entry) = @_;
	my $model	= $entry->driver->model;
	my $t_image	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/Image' );
	my $topic	= $entry->rdf_topic_of_type( $t_image );
	if ($topic) {
		my ($url)	= ($topic->is_resource) ? $topic->uri->as_string : '';
		if ($url) {
			$entry->image_url( $url );
		}
	}
});

MT::Entry->add_callback("pre_save", 4, $plugin_data, sub {
	my ($cb, $entry)	= @_;
	my $model	= $entry->driver->model;
	if (my $url = delete $entry->{'column_values'}{'image_url'}) {
		my $node	= $entry->get_rdf_node;
		my $t_image	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/Image' );
		my $p_topic	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/topic' );
		my $p_type	= RDF::Redland::Node->new_from_uri( "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
		
		# XXX should only remove topics of type foaf:Image
		my $st		= RDF::Redland::Statement->new( $node, $p_topic, undef );
		my @st		= $model->find_statements( $st );
		$model->remove_statement( $_ ) for (@st);
		
		my ($image)	= RDF::Redland::Node->new_from_uri( $url );
		my $tst		= RDF::Redland::Statement->new( $image, $p_type, $t_image );
		unless ($model->contains_statement( $tst )) {
			$model->add_statement( $tst );
		}
		$st->object( $image );
		unless ($model->contains_statement( $st )) {
			$model->add_statement( $st );
		}
	}
});


######################################################################

1;

__END__

=head1 REVISION HISTORY

 $Log: redland_cms_topic_image.pl,v $
 Revision 1.3  2005/04/04 10:12:45  greg
 - $VERSION now based on CVS revision

 Revision 1.2  2005/04/04 07:46:37  greg
 - moved topic_of_type() and topics() into MT::Entry namespace as
   rdf_topic_of_type() and rdf_topics()

 Revision 1.1  2005/04/03 01:43:30  greg
 - added redland_cms_topic_image.pl for foaf:Image topics
 - added stub plugin for Redland Template Tags


=head1 SEE ALSO

 L<MT::Plugin>

=head1 KNOWN BUGS

None.

=head1 TODO

None.

=head1 LICENSE

Copyright (c) 2005 Gregory Williams. All rights reserved. This program is free
software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=head1 AUTHOR

 Gregory Williams <gwilliams@cpan.org>

=cut
